@@ -14,16 +14,21 @@ def _assign_graph_outputs(outputs: Any, graph: NodeGraph) -> None:
       - dict        -> every leaf value (including nested dicts) must be BaseSocket
       - tuple       -> every item must be BaseSocket; length must match declared outputs
     """
-    from node_graph.socket import BaseSocket, NodeSocket, NodeSocketNamespace
+    from node_graph.socket import (
+        BaseSocket,
+        NodeSocket,
+        NodeSocketNamespace,
+        TaggedValue,
+    )
 
     def _ensure_all_sockets_in_dict(d: dict, path: str = "outputs") -> None:
         for k, v in d.items():
             subpath = f"{path}.{k}"
             if isinstance(v, dict):
                 _ensure_all_sockets_in_dict(v, path=subpath)
-            elif not isinstance(v, BaseSocket):
+            elif not isinstance(v, (BaseSocket, TaggedValue)):
                 raise TypeError(
-                    f"Invalid output at '{subpath}': expected BaseSocket, got {type(v).__name__}."
+                    f"Invalid output at '{subpath}': expected BaseSocket or TaggedValue, got {type(v).__name__}."
                 )
 
     if outputs is None:
@@ -34,19 +39,37 @@ def _assign_graph_outputs(outputs: Any, graph: NodeGraph) -> None:
             )
         return
 
-    elif isinstance(outputs, NodeSocket):
+    elif isinstance(outputs, (NodeSocket, TaggedValue)):
         # Single socket -> assign to first declared output slot
         graph.outputs[0] = outputs
 
     elif isinstance(outputs, NodeSocketNamespace):
+        # if this is a dynamic namespace, we should link the top-level outputs
+        if outputs._metadata.dynamic:
+            graph.outputs = outputs
+
         for socket in outputs:
-            # skip some built-in outputs, e.g., the exit_code
             if (
                 socket._name not in graph.outputs
                 and not graph.outputs._metadata.dynamic
             ):
-                # Not declared and not dynamic: silently skip (or log if you prefer)
-                continue
+                if socket._name.startswith("_"):
+                    # Allow internal names to pass through even if not declared.
+                    # This is useful for internal sockets that don't need
+                    # to be exposed as a Graph output.
+                    continue
+                raise ValueError(
+                    "Output socket name "
+                    f"'{socket._name}' not declared in Graph node outputs.\n\n"
+                    "How to fix:\n"
+                    "1) Make graph outputs dynamic if you want to return arbitrary names:\n"
+                    "   • `outputs = dynamic(Any)`\n"
+                    "     (then returning arbitrary sockets is allowed)\n"
+                    "2) Declare this output explicitly on the Graph:\n"
+                    f"   • e.g. `outputs = namespace({socket._name}=Any, <some_other_name>=Any)`\n"
+                    "3) Expose task outputs instead of inventing new names:\n"
+                    f"   • e.g. `outputs = namespace({socket._name}=some_task.outputs, <some_other_name>=some_task.outputs)`\n"
+                )
             graph.outputs[socket._name] = socket
 
     elif isinstance(outputs, dict):
@@ -61,9 +84,9 @@ def _assign_graph_outputs(outputs: Any, graph: NodeGraph) -> None:
                 f"Graph node outputs {len(graph.outputs)}."
             )
         for i, output in enumerate(outputs):
-            if not isinstance(output, BaseSocket):
+            if not isinstance(output, (BaseSocket, TaggedValue)):
                 raise TypeError(
-                    f"Invalid output at index {i}: expected Socket, got {type(output).__name__}."
+                    f"Invalid output at index {i}: expected Socket or TaggedValue, got {type(output).__name__}."
                 )
         outputs_dict = {
             graph.outputs[i]._name: output for i, output in enumerate(outputs)
@@ -73,7 +96,7 @@ def _assign_graph_outputs(outputs: Any, graph: NodeGraph) -> None:
     else:
         raise TypeError(
             f"Unsupported output type {type(outputs).__name__}. Must be one of "
-            "Socket, dict, tuple, or None."
+            "Socket, TaggedValue, dict, tuple, or None."
         )
 
 
@@ -106,7 +129,7 @@ def materialize_graph(
         inputs = prepare_function_inputs(func, *args, **merged)
         graph.graph_inputs.set_inputs(inputs)
         tag_socket_value(graph.inputs)
-        inputs = graph.inputs._value
-        raw = func(**graph.inputs._value)
+        inputs = graph.inputs._collect_values(raw=False)
+        raw = func(**inputs)
         _assign_graph_outputs(raw, graph)
         return graph
